<?php

namespace App\Http\Controllers\payment;

use App\Http\Controllers\Controller;
use App\Models\Log as ModelsLog;
use App\Services\Error\SetLogs;
use App\Services\Gateway;
use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request as Psr7Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Proxy\Http\Request as ProxyRequest;
use Proxy\Proxy;


class PayController extends Controller
{
    protected $service, $logs;

    public function __construct()
    {
        $this->service = new Gateway;

        $this->logs = new SetLogs();
    }

    public function prepare(Request $request)
    {
        // request becomes a proxy and send to laravel then response of that will be use in continue.
        $validator = Validator::make(
            $request->all(),
            [
                'amount' => 'required',
            ]
        );
        if ($validator->fails()) {
            $errors = $validator->errors()->all();
            $count = count([array_shift($errors)]);
            $pluralized = $count === 1 ? 'error' : 'errors';
            $extendText = count($errors) >= 1 ? (' (and ' . $count . ' more ' . $pluralized . ')') : '';
            return response()->json([
                'status' => 'error',
                'message' => $validator->errors()->first() . $extendText,
                'messages' => $validator->errors()->all(),
            ], 400);
        }

        try {

            $client = new Client();
            $headers = [
                'Authorization' => $request->header('Authorization'),
                'PROXY-TOKEN' => config('proxy.reseller_token'),
                "Accept" => "application/json"
            ];
            $options = [
                'form_params' => $request->all()
            ];
            $request = new Psr7Request('POST', "https://" . config('proxy.target_url') .  '/api/v3/pay', $headers);
            $res = $client->sendAsync($request, $options)->wait();

            $parsedData = json_decode($res->getBody()->getContents(), true);
            if ($parsedData['status'] != 'success') {
                throw new \Exception($parsedData['message']);
            }

            $prepare = $this->service->prepare($parsedData['data']);

            throw_if(!isset($prepare['url']) || empty($prepare['url']),  Exception::class, 'expected send url and missing that');

            return response()->json([
                'status' => 'success',
                'data' => [
                    'url' => $prepare['url']
                ]
            ], 200);
        } catch (\Throwable $th) {
            $this->logs->setLogs(ModelsLog::LEVEL['prepare_payment'], $th->getMessage(), $th->getTraceAsString());
            Log::error('can not prepare payment and create payment url because : ' . $th->getMessage() . $th->getTraceAsString());

            return response()->json([
                'status' => 'error',
                'message' => $th->getMessage()
            ], 500);
        }
    }

    public function payment()
    {
        return $this->service->pay();
    }

    public function verify()
    {
        return $this->service->verify();
    }
}
